@@ -2,18 +2,19 @@ import logging
 import functools
 import json
 import uuid
+from inspect import signature
 
 import bizerror
+from fastutils import jsonutils
 from fastutils import rsautils
 from fastutils import strutils
 from fastutils import funcutils
 from fastutils import typingutils
 from fastutils import cipherutils
 
-from django.http import JsonResponse
+from django.http import HttpResponse
 from django.conf import settings
 
-from .utils import SimpleJsonEncoder
 from .utils import get_request_data
 from .pack import SimpleJsonResultPacker
 from .pack import SafeJsonResultPacker
@@ -22,10 +23,16 @@ logger = logging.getLogger(__name__)
 
 simple_result_packer = SimpleJsonResultPacker()
 
+class _Null(object):
+    pass
+_Null = _Null()
+
 def setup_result_packer(packer):
     global simple_result_packer
     simple_result_packer = packer
 
+class DoNotPack(RuntimeError):
+    pass
 
 class ApiviewDecorator(object):
     """Decorator base class.
@@ -40,11 +47,10 @@ class ApiviewDecorator(object):
     def main(self, _django_apiview_request, func):
         return _django_apiview_request.process(func)
 
-
 class ApiviewRequest(object):
     """Process class of apiview.
     """
-    
+
     def __init__(self, request, defaults, **kwargs):
         self._apiview_decorator = False
         self.request = request
@@ -53,12 +59,16 @@ class ApiviewRequest(object):
         self.data.update(defaults)
         self.data.update(get_request_data(self.request, self.kwargs))
         self.data["_django_apiview_request"] = self
+        self.data["_django_apiview_response"] = HttpResponse(content_type="application/json;charset=UTF-8")
 
     def process(self, func):
         if hasattr(func, "_apiview_decorator") and getattr(func, "_apiview_decorator"):
             return func(_django_apiview_request=self)
         else:
-            return funcutils.call_with_inject(func, self.data)
+            logger.debug("ApiviewRequest calling funcutils.call_with_inject: func={func} data={data}".format(func=signature(func), data=self.data))
+            result = funcutils.call_with_inject(func, self.data)
+            logger.debug("ApiviewRequest call funcutils.call_with_inject done, result={result}".format(result=result))
+            return result
 
 class Apiview(object):
     def __init__(self, packer, preload_plugins=None, extra_parameters=None, **kwargs):
@@ -94,12 +104,19 @@ class Apiview(object):
                     target_func = plugin(target_func)
                 result = _django_apiview_request.process(target_func)
                 package = self.packer.pack_result(result, **_django_apiview_request.data)
+                package_json = jsonutils.simple_json_dumps(package, ensure_ascii=False, allow_nan=True, sort_keys=True)
+                _django_apiview_request.data["_django_apiview_response"].write(package_json)
+            except DoNotPack:
+                # do not need to do result pack
+                pass
             except Exception as error:
                 logger.exception("apiview process failed: {}".format(str(error)))
                 if not isinstance(error, bizerror.BizErrorBase):
                     error = bizerror.BizError(error)
                 package = self.packer.pack_error(error, **_django_apiview_request.data)
-            return JsonResponse(package, encoder=SimpleJsonEncoder, json_dumps_params={"ensure_ascii": False, "allow_nan": True, "sort_keys": True})
+                package_json = jsonutils.simple_json_dumps(package, ensure_ascii=False, allow_nan=True, sort_keys=True)
+                _django_apiview_request.data["_django_apiview_response"].write(package_json)
+            return _django_apiview_request.data["_django_apiview_response"]
         wrapper.csrf_exempt = True
         return functools.wraps(func)(wrapper)
 
@@ -207,12 +224,25 @@ class rsa_decrypt(ApiviewDecorator):
     def main(self, _django_apiview_request, func):
         if self.field in _django_apiview_request.data:
             field_value = _django_apiview_request.data[self.field]
-            field_data = rsautils.smart_get_binary_data(field_value)
-            plain_data = rsautils.decrypt(field_data, self.private_key)
-            plain_text = plain_data.decode("utf-8")
-            _django_apiview_request.data[self.field] = plain_text
+            if field_value:
+                field_data = rsautils.smart_get_binary_data(field_value)
+                plain_data = rsautils.decrypt(field_data, self.private_key)
+                plain_text = plain_data.decode("utf-8")
+                _django_apiview_request.data[self.field] = plain_text
         return super().main(_django_apiview_request, func)
 
+class cookie_variable(ApiviewDecorator):
+    """Get variable from cookies.
+    """
+    def __init__(self, variable_name, cookie_name):
+        self.variable_name = variable_name
+        self.cookie_name = cookie_name
+    
+    def main(self, _django_apiview_request, func):
+        value = _django_apiview_request.request.COOKIES.get(self.cookie_name, _Null)
+        if not value is _Null:
+            _django_apiview_request.data[self.variable_name] = value
+        return super().main(_django_apiview_request, func)
 
 class meta_variable(ApiviewDecorator):
     """Get variable from meta.
@@ -222,8 +252,9 @@ class meta_variable(ApiviewDecorator):
         self.meta_name = meta_name
     
     def main(self, _django_apiview_request, func):
-        value = _django_apiview_request.request.META.get(self.meta_name, None)
-        _django_apiview_request.data[self.variable_name] = value
+        value = _django_apiview_request.request.META.get(self.meta_name, _Null)
+        if not value is _Null:
+            _django_apiview_request.data[self.variable_name] = value
         return super().main(_django_apiview_request, func)
 
 class cache(ApiviewDecorator):
@@ -332,7 +363,7 @@ class cache(ApiviewDecorator):
                     funcutils.chain(strutils.none_to_empty_string, strutils.strip_string),
                     **_django_apiview_request.data, **kwargs,
                     )
-            result_text = json.dumps(result, cls=SimpleJsonEncoder, allow_nan=True, sort_keys=True)
+            result_text = jsonutils.simple_json_dumps(result, allow_nan=True, sort_keys=True)
             cache.set(key, result_text)
             if expire:
                 cache.expire(key, expire) # Old version redis don't support add ttl while doing set key value, so do set ttl separately
@@ -385,7 +416,6 @@ def safe_apiview(packer_class=SafeJsonResultPacker, **kwargs):
         extra_parameters=kwargs,
         )
 
-
 DEFAULT_RANDOM_DJANGO_APIVIEW_ACLKEY = str(uuid.uuid4())
 logger.info("Default random DJANGO_APIVIEW_ACLKEY={0}".format(DEFAULT_RANDOM_DJANGO_APIVIEW_ACLKEY))
 
@@ -408,7 +438,6 @@ class check_aclkey(ApiviewDecorator):
             aclkey = funcutils.call_with_inject(self.aclkey, _django_apiview_request.data)
         return self.aclkey == aclkey
 
-
 class string_length_limit(ApiviewDecorator):
     """Check string parameter length.
     """
@@ -428,5 +457,3 @@ class string_length_limit(ApiviewDecorator):
             if value_length > self.max_length:
                 raise bizerror.StringTooLong(message=self.string_too_long_error_message, max_length=self.max_length)
         return super().main(_django_apiview_request, func)
-
-
